@@ -203,9 +203,7 @@ static std::vector<ElfPhdr<E>> create_phdr(Context<E> &ctx) {
   // The ELF spec says that "loadable segment entries in the program
   // header table appear in ascending order, sorted on the p_vaddr
   // member".
-  sort(chunks, [](Chunk<E> *a, Chunk<E> *b) {
-    return a->shdr.sh_addr < b->shdr.sh_addr;
-  });
+  ranges::stable_sort(chunks, {}, [](Chunk<E> *x) { return x->shdr.sh_addr; });
 
   // Create a PT_PHDR for the program header itself.
   if (ctx.phdr && (ctx.phdr->shdr.sh_flags & SHF_ALLOC))
@@ -657,6 +655,12 @@ static std::vector<Word<E>> create_dynamic_section(Context<E> &ctx) {
   for (std::string_view str : ctx.arg.auxiliary)
     define(DT_AUXILIARY, ctx.dynstr->find_string(str));
 
+  if (!ctx.arg.audit.empty())
+    define(DT_AUDIT, ctx.dynstr->find_string(ctx.arg.audit));
+
+  if (!ctx.arg.depaudit.empty())
+    define(DT_DEPAUDIT, ctx.dynstr->find_string(ctx.arg.depaudit));
+
   for (std::string_view str : ctx.arg.filter)
     define(DT_FILTER, ctx.dynstr->find_string(str));
 
@@ -830,20 +834,6 @@ void DynamicSection<E>::copy_buf(Context<E> &ctx) {
   write_vector(ctx.buf + this->shdr.sh_offset, contents);
 }
 
-template <typename T>
-static std::vector<std::span<T>> split(std::vector<T> &input, i64 unit) {
-  std::span<T> span(input);
-  std::vector<std::span<T>> vec;
-
-  while (span.size() >= unit) {
-    vec.push_back(span.subspan(0, unit));
-    span = span.subspan(unit);
-  }
-  if (!span.empty())
-    vec.push_back(span);
-  return vec;
-}
-
 // Assign offsets to OutputSection members
 template <typename E>
 void OutputSection<E>::compute_section_size(Context<E> &ctx) {
@@ -861,6 +851,7 @@ void OutputSection<E>::compute_section_size(Context<E> &ctx) {
     std::span<InputSection<E> *> members;
     i64 size = 0;
     i64 offset = 0;
+    i64 align = 1;
   };
 
   std::vector<Group> groups;
@@ -872,26 +863,28 @@ void OutputSection<E>::compute_section_size(Context<E> &ctx) {
     m = m.subspan(sz);
   }
 
-  tbb::parallel_for_each(groups, [](Group &group) {
+  tbb::parallel_for_each(groups, [](Group &g) {
     i64 off = 0;
-    for (InputSection<E> *isec : group.members)
+    for (InputSection<E> *isec : g.members) {
       off = align_to(off, 1 << isec->p2align) + isec->sh_size;
-    group.size = off;
+      g.align = std::max<i64>(g.align, 1 << isec->p2align);
+    }
+    g.size = off;
   });
 
   i64 off = 0;
-  for (i64 i = 0; i < groups.size(); i++) {
-    off = align_to(off, shdr.sh_addralign);
-    groups[i].offset = off;
-    off += groups[i].size;
+  for (Group &g : groups) {
+    off = align_to(off, g.align);
+    g.offset = off;
+    off += g.size;
   }
 
   shdr.sh_size = off;
 
   // Assign offsets to input sections.
-  tbb::parallel_for_each(groups, [](Group &group) {
-    i64 off = group.offset;
-    for (InputSection<E> *isec : group.members) {
+  tbb::parallel_for_each(groups, [](Group &g) {
+    i64 off = g.offset;
+    for (InputSection<E> *isec : g.members) {
       off = align_to(off, 1 << isec->p2align);
       isec->offset = off;
       off += isec->sh_size;
@@ -928,7 +921,7 @@ void OutputSection<E>::copy_buf(Context<E> &ctx) {
       P -= delta;
     }
 
-    auto dynrel = [&](i64 ty, i64 idx, u64 val) {
+    auto write = [&](i64 ty, i64 idx, u64 val) {
       *rel++ = ElfRel<E>(P, ty, idx, val);
       if (ctx.arg.apply_dynamic_relocs)
         *(Word<E> *)loc = val;
@@ -940,14 +933,14 @@ void OutputSection<E>::copy_buf(Context<E> &ctx) {
       *(Word<E> *)loc = S + A;
       break;
     case ABS_REL_BASEREL:
-      dynrel(E::R_RELATIVE, 0, S + A);
+      write(E::R_RELATIVE, 0, S + A);
       break;
     case ABS_REL_IFUNC:
       if constexpr (supports_ifunc<E>)
-        dynrel(E::R_IRELATIVE, 0, sym.get_addr(ctx, NO_PLT) + A);
+        write(E::R_IRELATIVE, 0, sym.get_addr(ctx, NO_PLT) + A);
       break;
     case ABS_REL_DYNREL:
-      dynrel(E::R_ABS, sym.get_dynsym_idx(ctx), A);
+      write(E::R_ABS, sym.get_dynsym_idx(ctx), A);
       break;
     }
   }
@@ -960,8 +953,8 @@ void OutputSection<E>::write_to(Context<E> &ctx, u8 *buf) {
     InputSection<E> &isec = *members[i];
     isec.write_to(ctx, buf + isec.offset);
 
-    // Clear trailing padding. We write trap or nop instructions for
-    // an executable segment so that a disassembler wouldn't try to
+    // Clear trailing padding. We write trap instructions for an
+    // executable segment so that a disassembler wouldn't try to
     // disassemble garbage as instructions.
     u64 this_end = isec.offset + isec.sh_size;
     u64 next_start;
@@ -973,9 +966,18 @@ void OutputSection<E>::write_to(Context<E> &ctx, u8 *buf) {
     u8 *loc = buf + this_end;
     i64 size = next_start - this_end;
 
+    auto fill = [&]<size_t N>(const u8 (&filler)[N]) {
+      for (i64 i = 0; i + N <= size; i += N)
+        memcpy(loc + i, filler, N);
+    };
+
     if (this->shdr.sh_flags & SHF_EXECINSTR) {
-      for (i64 i = 0; i + sizeof(E::filler) <= size; i += sizeof(E::filler))
-        memcpy(loc + i, E::filler, sizeof(E::filler));
+      // s390x's old CRT files use NOP slides in .init and .fini.
+      // https://sourceware.org/bugzilla/show_bug.cgi?id=31042
+      if (is_s390x<E> && (this->name == ".init" || this->name == ".fini"))
+        fill({ 0x07, 0x00 }); // nopr
+      else
+        fill(E::trap);
     } else {
       memset(loc, 0, size);
     }
@@ -1008,9 +1010,8 @@ void OutputSection<E>::write_to(Context<E> &ctx, u8 *buf) {
 // the .rel.dyn section). A bitmap has LSB 1.
 template <typename E>
 static std::vector<u64> encode_relr(std::span<u64> pos) {
-  assert(std::all_of(pos.begin(), pos.end(),
-                     [](u64 x) { return x % sizeof(Word<E>) == 0; }));
-  assert(std::is_sorted(pos.begin(), pos.end()));
+  assert(ranges::all_of(pos, [](u64 x) { return x % sizeof(Word<E>) == 0; }));
+  assert(ranges::is_sorted(pos));
 
   std::vector<u64> vec;
   i64 num_bits = E::is_64 ? 63 : 31;
@@ -1051,6 +1052,16 @@ static AbsRelKind get_abs_rel_kind(Context<E> &ctx, Symbol<E> &sym) {
   return ABS_REL_DYNREL;
 }
 
+template <typename E>
+static bool is_absrel(const ElfRel<E> &r) {
+  // On ARM32, R_ARM_TARGET1 is typically used for entries in .init_array
+  // and is interpreted as either ABS32 or REL32 depending on the target.
+  // All targets we support handle it as if it were a ABS32.
+  if constexpr (is_arm32<E>)
+    return r.r_type == E::R_ABS || r.r_type == R_ARM_TARGET1;
+  return r.r_type == E::R_ABS;
+}
+
 // Scan word-size absolute relocations (e.g. R_X86_64_64). This is
 // separated from scan_relocations() because only such relocations can
 // be promoted to dynamic relocations.
@@ -1062,7 +1073,7 @@ void OutputSection<E>::scan_abs_relocations(Context<E> &ctx) {
   tbb::parallel_for((i64)0, (i64)members.size(), [&](i64 i) {
     InputSection<E> *isec = members[i];
     for (const ElfRel<E> &r : isec->get_rels(ctx))
-      if (r.r_type == E::R_ABS)
+      if (is_absrel(r))
         shards[i].push_back(AbsRel<E>{isec, r.r_offset, isec->file.symbols[r.r_sym],
                                       get_addend(*isec, r)});
   });
@@ -1160,6 +1171,9 @@ void OutputSection<E>::populate_symtab(Context<E> &ctx) {
     u8 *strtab_base = ctx.buf + ctx.strtab->shdr.sh_offset;
     u8 *strtab = strtab_base + this->strtab_offset;
 
+    memset(esym, 0, this->num_local_symtab * sizeof(ElfSym<E>));
+    memset(strtab, 0, this->strtab_size);
+
     auto write_esym = [&](u64 addr, i64 st_name) {
       memset(esym, 0, sizeof(*esym));
       esym->st_name = st_name;
@@ -1172,7 +1186,7 @@ void OutputSection<E>::populate_symtab(Context<E> &ctx) {
     for (std::unique_ptr<Thunk<E>> &thunk : thunks) {
       for (i64 i = 0; i < thunk->symbols.size(); i++) {
         Symbol<E> &sym = *thunk->symbols[i];
-        u64 addr = thunk->get_addr(i);
+        u64 addr = thunk->get_addr() + thunk->offsets[i];
 
         write_esym(addr, strtab - strtab_base);
 
@@ -1299,18 +1313,16 @@ static std::vector<GotEntry<E>> get_got_entries(Context<E> &ctx) {
       }
     }
 
-    // If a symbol is imported, let the dynamic linker to resolve it.
     if (sym->is_imported) {
+      // If a symbol is imported, let the dynamic linker to resolve it.
       add({idx, 0, E::R_GLOB_DAT, sym});
-      continue;
-    }
-
-    // If we know an address at link-time, fill that GOT entry now.
-    // It may need a base relocation, though.
-    if (ctx.arg.pic && sym->is_relative())
+    } else if (ctx.arg.pic && sym->is_relative()) {
+      // We know the symbol's address, but it needs a base relocation.
       add({idx, sym->get_addr(ctx, NO_PLT), E::R_RELATIVE});
-    else
+    } else {
+      // We know the symbol's exact run-time address at link-time.
       add({idx, sym->get_addr(ctx, NO_PLT)});
+    }
   }
 
   // Create GOT entries for TLVs.
@@ -1420,23 +1432,20 @@ void GotSection<E>::copy_buf(Context<E> &ctx) {
                        ent.sym ? ent.sym->get_dynsym_idx(ctx) : 0,
                        ent.val);
 
-    // A single TLSDESC relocation fixes two consecutive GOT slots
-    // where one slot holds a function pointer and the other an
-    // argument to the function. An addend should be applied not to
-    // the function pointer but to the function argument, which is
-    // usually stored to the second slot.
-    //
-    // ARM32 employs the inverted layout for some reason, so an
-    // addend is applied to the first slot.
-    bool is_tlsdesc = false;
-    if constexpr (supports_tlsdesc<E> && !is_arm32<E>)
-      is_tlsdesc = (ent.r_type == E::R_TLSDESC);
-
     if (ctx.arg.apply_dynamic_relocs) {
-      if (is_tlsdesc)
-        buf[ent.idx + 1] = ent.val;
-      else
-        buf[ent.idx] = ent.val;
+      // A single TLSDESC relocation fixes two consecutive GOT slots
+      // where one slot holds a function pointer and the other an
+      // argument to the function. An addend should be applied not to
+      // the function pointer but to the function argument, which is
+      // usually stored to the second slot.
+      //
+      // ARM32 employs the inverted layout for some reason, so an
+      // addend is applied to the first slot.
+      i64 i = ent.idx;
+      if constexpr (supports_tlsdesc<E> && !is_arm32<E>)
+        if (ent.r_type == E::R_TLSDESC)
+          i = ent.idx + 1;
+      buf[i] = ent.val;
     }
   }
 }
@@ -1718,14 +1727,15 @@ static u64 get_symbol_size(Symbol<E> &sym) {
   if constexpr (is_riscv<E> || is_loongarch<E>)
     if (esym.st_size)
       if (InputSection<E> *isec = sym.get_input_section())
-        return esym.st_size + esym.st_value - sym.value -
-               get_r_delta(*isec, esym.st_value + esym.st_size);
+        if (isec->shdr().sh_flags & SHF_EXECINSTR)
+          return esym.st_size + esym.st_value - sym.value -
+                 get_r_delta(*isec, esym.st_value + esym.st_size);
   return esym.st_size;
 }
 
 template <typename E>
-ElfSym<E> to_output_esym(Context<E> &ctx, Symbol<E> &sym, u32 st_name,
-                         U32<E> *shn_xindex) {
+std::optional<ElfSym<E>>
+to_output_esym(Context<E> &ctx, Symbol<E> &sym, u32 st_name, U32<E> *shn_xindex) {
   ElfSym<E> esym;
   memset(&esym, 0, sizeof(esym));
 
@@ -1834,7 +1844,8 @@ ElfSym<E> to_output_esym(Context<E> &ctx, Symbol<E> &sym, u32 st_name,
   if (0 <= shndx && shndx < SHN_LORESERVE) {
     esym.st_shndx = shndx;
   } else if (SHN_LORESERVE <= shndx) {
-    assert(shn_xindex);
+    if (!shn_xindex)
+      return {};
     esym.st_shndx = SHN_XINDEX;
     *shn_xindex = shndx;
   }
@@ -1868,7 +1879,17 @@ void DynsymSection<E>::copy_buf(Context<E> &ctx) {
 
   for (i64 i = 1; i < symbols.size(); i++) {
     Symbol<E> &sym = *symbols[i];
-    buf[sym.get_dynsym_idx(ctx)] = to_output_esym(ctx, sym, offset, nullptr);
+
+    std::optional<ElfSym<E>> esym = to_output_esym(ctx, sym, offset, nullptr);
+    if (!esym) {
+      Error(ctx) << ctx.arg.output
+                 << ": .dynsym: too many output sections: "
+                 << (ctx.shdr->shdr.sh_size / sizeof(ElfShdr<E>))
+                 << " requested, but ELF allows at most 65279";
+      return;
+    }
+
+    buf[sym.get_dynsym_idx(ctx)] = *esym;
     offset += sym.name().size() + 1;
   }
 }
@@ -1980,7 +2001,7 @@ get_merged_output_name(Context<E> &ctx, std::string_view name, u64 flags,
                        i64 entsize, i64 addralign) {
   if (ctx.arg.relocatable && !ctx.arg.relocatable_merge_sections)
     return name;
-  if (ctx.arg.unique && ctx.arg.unique->match(name))
+  if (!ctx.arg.unique.empty() && ctx.arg.unique.find(name) != -1)
     return name;
 
   // GCC seems to create sections named ".rodata.strN.<mangled-symbol-name>.M"
@@ -2092,7 +2113,7 @@ static void add_comment_strings(Context<E> &ctx) {
   };
 
   // Add an identification string to .comment.
-  add(get_mold_version());
+  add(mold_version);
 
   // Embed command line arguments for debugging.
   char *env = getenv("MOLD_DEBUG");
@@ -2130,43 +2151,54 @@ void MergedSection<E>::compute_section_size(Context<E> &ctx) {
   if (!resolved)
     resolve(ctx);
 
-  std::vector<i64> sizes(map.NUM_SHARDS);
+  std::vector<i64> sizes(map.NUM_SHARDS * 2);
 
   tbb::parallel_for((i64)0, map.NUM_SHARDS, [&](i64 i) {
     using Entry = typename decltype(map)::Entry;
     std::vector<Entry *> entries = map.get_sorted_entries(i);
-    i64 offset = 0;
+
+    i64 off1 = 0;
+    i64 off2 = 0;
 
     for (Entry *ent : entries) {
       SectionFragment<E> &frag = ent->value;
       if (frag.is_alive) {
-        offset = align_to(offset, 1 << frag.p2align);
-        frag.offset = offset;
-        offset += ent->keylen;
+        if (frag.is_32bit) {
+          off1 = align_to(off1, 1 << frag.p2align);
+          frag.offset = off1;
+          off1 += ent->keylen;
+        } else {
+          off2 = align_to(off2, 1 << frag.p2align);
+          frag.offset = off2;
+          off2 += ent->keylen;
+        }
       }
     }
-    sizes[i] = offset;
+
+    sizes[i] = off1;
+    sizes[i + map.NUM_SHARDS] = off2;
   });
 
   i64 shard_size = map.nbuckets / map.NUM_SHARDS;
-  shard_offsets.resize(map.NUM_SHARDS + 1);
+  shard_offsets.resize(sizes.size() + 1);
 
-  for (i64 i = 1; i < map.NUM_SHARDS + 1; i++)
+  for (i64 i = 1; i < sizes.size() + 1; i++)
     shard_offsets[i] =
       align_to(shard_offsets[i - 1] + sizes[i - 1], this->shdr.sh_addralign);
 
+  this->shdr.sh_size = shard_offsets.back();
+
   tbb::parallel_for((i64)1, map.NUM_SHARDS, [&](i64 i) {
     for (i64 j = shard_size * i; j < shard_size * (i + 1); j++) {
       SectionFragment<E> &frag = map.entries[j].value;
-      if (frag.is_alive)
-        frag.offset += shard_offsets[i];
+      if (frag.is_alive) {
+        if (frag.is_32bit)
+          frag.offset += shard_offsets[i];
+        else
+          frag.offset += shard_offsets[i + map.NUM_SHARDS];
+      }
     }
   });
-
-  this->shdr.sh_size = shard_offsets[map.NUM_SHARDS];
-
-  if (this->shdr.sh_size > UINT32_MAX)
-    Fatal(ctx) << this->name << ": output section too large";
 }
 
 template <typename E>
@@ -2182,8 +2214,11 @@ void MergedSection<E>::write_to(Context<E> &ctx, u8 *buf) {
     // There might be gaps between strings to satisfy alignment requirements.
     // If that's the case, we need to zero-clear them.
     if (this->shdr.sh_addralign > 1 &&
-        this->shdr.sh_addralign != this->shdr.sh_entsize)
+        this->shdr.sh_addralign != this->shdr.sh_entsize) {
       memset(buf + shard_offsets[i], 0, shard_offsets[i + 1] - shard_offsets[i]);
+      i64 j = map.NUM_SHARDS + i;
+      memset(buf + shard_offsets[j], 0, shard_offsets[j + 1] - shard_offsets[j]);
+    }
 
     // Copy strings
     for (i64 j = shard_size * i; j < shard_size * (i + 1); j++)
@@ -2433,21 +2468,30 @@ void EhFrameRelocSection<E>::copy_buf(Context<E> &ctx) {
 
 template <typename E>
 void CopyrelSection<E>::add_symbol(Context<E> &ctx, Symbol<E> *sym) {
+  assert(!ctx.arg.shared);
   if (sym->has_copyrel)
     return;
 
-  assert(!ctx.arg.shared);
-  assert(sym->file->is_dso);
+  if (!sym->file->is_dso) {
+    assert(sym->esym().is_undef_weak());
+    Error(ctx) << *sym->file << ": cannot create a copy relocation for "
+               << *sym <<"; recompile with -fPIE or -fPIC";
+    return;
+  }
 
-  if (sym->esym().st_visibility == STV_PROTECTED)
+  if (sym->esym().st_visibility == STV_PROTECTED) {
     Error(ctx) << *sym->file
                << ": cannot create a copy relocation for protected symbol '"
                << *sym << "'; recompile with -fPIC";
+    return;
+  }
 
-  if (!ctx.arg.z_copyreloc)
+  if (!ctx.arg.z_copyreloc) {
     Error(ctx) << "-z nocopyreloc: " << *sym->file
                << ": cannot create a copy relocation for symbol '" << *sym
                << "'; recompile with -fPIC";
+    return;
+  }
 
   symbols.push_back(sym);
 
@@ -2538,9 +2582,8 @@ void VerneedSection<E>::construct(Context<E> &ctx) {
   if (syms.empty())
     return;
 
-  sort(syms, [](Symbol<E> *a, Symbol<E> *b) {
-    return std::tuple(((SharedFile<E> *)a->file)->soname, a->ver_idx) <
-           std::tuple(((SharedFile<E> *)b->file)->soname, b->ver_idx);
+  ranges::stable_sort(syms, {}, [](Symbol<E> *x) {
+    return std::tuple{((SharedFile<E> *)x->file)->soname, x->ver_idx};
   });
 
   // Resize .gnu.version
@@ -2621,14 +2664,21 @@ void VerdefSection<E>::construct(Context<E> &ctx) {
   if (ctx.arg.version_definitions.empty())
     return;
 
+  // Handle --default-symver
+  if (ctx.arg.default_symver)
+    for (Symbol<E> *sym : ctx.dynsym->symbols)
+      if (sym && !sym->file->is_dso)
+        if (u16 ver = sym->ver_idx;
+            ver == VER_NDX_GLOBAL || ver == VER_NDX_UNSPECIFIED)
+          sym->ver_idx = VER_NDX_LAST_RESERVED + 1;
+
   // Resize .gnu.version and write to it
   ctx.versym->contents.resize(ctx.dynsym->symbols.size(), VER_NDX_GLOBAL);
   ctx.versym->contents[0] = VER_NDX_LOCAL;
 
-  for (i64 i = 1; i < ctx.dynsym->symbols.size(); i++)
-    if (Symbol<E> &sym = *ctx.dynsym->symbols[i];
-        !sym.file->is_dso && sym.ver_idx != VER_NDX_UNSPECIFIED)
-      ctx.versym->contents[sym.get_dynsym_idx(ctx)] = sym.ver_idx;
+  for (Symbol<E> *sym : ctx.dynsym->symbols)
+    if (sym && !sym->file->is_dso && sym->ver_idx != VER_NDX_UNSPECIFIED)
+      ctx.versym->contents[sym->get_dynsym_idx(ctx)] = sym->ver_idx;
 
   // Allocate a buffer for .gnu.version_d and write to it
   contents.resize((sizeof(ElfVerdef<E>) + sizeof(ElfVerdaux<E>)) *
@@ -2738,10 +2788,6 @@ void NotePropertySection<E>::update_shdr(Context<E> &ctx) {
   std::map<u32, u32> map;
 
   for (u32 key : keys) {
-    auto has_key = [&](ObjectFile<E> *file) {
-      return file->gnu_properties.contains(key);
-    };
-
     if (GNU_PROPERTY_X86_UINT32_AND_LO <= key &&
         key <= GNU_PROPERTY_X86_UINT32_AND_HI) {
       // An AND feature is set if all input objects have the property and
@@ -2758,7 +2804,11 @@ void NotePropertySection<E>::update_shdr(Context<E> &ctx) {
                key <= GNU_PROPERTY_X86_UINT32_OR_AND_HI) {
       // An OR-AND feature is set if all input object files have the property
       // and some of them has the feature.
-      if (std::all_of(files.begin(), files.end(), has_key))
+      auto has_key = [&](ObjectFile<E> *file) {
+        return file->gnu_properties.contains(key);
+      };
+
+      if (ranges::all_of(files, has_key))
         for (ObjectFile<E> *file : files)
           map[key] |= get_value(file, key);
     }
@@ -2797,42 +2847,36 @@ void NotePropertySection<E>::copy_buf(Context<E> &ctx) {
 
 template <typename E>
 CompressedSection<E>::CompressedSection(Context<E> &ctx, Chunk<E> &chunk) {
-  assert(chunk.name.starts_with(".debug"));
-  this->name = chunk.name;
-  this->is_compressed = true;
+  // Allocate a temporary buffer to write uncompressed contents. Note
+  // that we use u8[] instead of std::vector<u8> to avoid the cost of
+  // zero-initialization, as sh_size can be very large.
+  std::unique_ptr<u8[]> buf(new u8[chunk.shdr.sh_size]);
 
-  this->uncompressed_data.resize(chunk.shdr.sh_size);
-  u8 *buf = this->uncompressed_data.data();
+  // Write uncompressed contents and then compress them
+  chunk.write_to(ctx, buf.get());
 
-  chunk.write_to(ctx, buf);
-
-  switch (ctx.arg.compress_debug_sections) {
-  case COMPRESS_ZLIB:
-    chdr.ch_type = ELFCOMPRESS_ZLIB;
-    compressor.reset(new ZlibCompressor(buf, chunk.shdr.sh_size));
-    break;
-  case COMPRESS_ZSTD:
-    chdr.ch_type = ELFCOMPRESS_ZSTD;
-    compressor.reset(new ZstdCompressor(buf, chunk.shdr.sh_size));
-    break;
-  default:
-    unreachable();
-  }
+  if (ctx.arg.compress_debug_sections == ELFCOMPRESS_ZLIB)
+    compressor.reset(new ZlibCompressor(buf.get(), chunk.shdr.sh_size));
+  else
+    compressor.reset(new ZstdCompressor(buf.get(), chunk.shdr.sh_size));
 
+  // Compute header field values
+  chdr.ch_type = ctx.arg.compress_debug_sections;
   chdr.ch_size = chunk.shdr.sh_size;
   chdr.ch_addralign = chunk.shdr.sh_addralign;
 
+  this->name = chunk.name;
+  this->shndx = chunk.shndx;
+  this->is_compressed = true;
+
   this->shdr = chunk.shdr;
   this->shdr.sh_flags |= SHF_COMPRESSED;
   this->shdr.sh_addralign = 1;
   this->shdr.sh_size = sizeof(chdr) + compressor->compressed_size;
-  this->shndx = chunk.shndx;
 
-  // We don't need to keep the original data unless --gdb-index is given.
-  if (!ctx.arg.gdb_index) {
-    this->uncompressed_data.clear();
-    this->uncompressed_data.shrink_to_fit();
-  }
+  // We can discard the uncompressed contents unless --gdb-index is given
+  if (ctx.arg.gdb_index)
+    this->uncompressed_data = std::move(buf);
 }
 
 template <typename E>
@@ -3015,6 +3059,8 @@ template class GnuDebuglinkSection<E>;
 template Chunk<E> *find_chunk(Context<E> &, u32);
 template Chunk<E> *find_chunk(Context<E> &, std::string_view);
 template i64 to_phdr_flags(Context<E> &ctx, Chunk<E> *chunk);
-template ElfSym<E> to_output_esym(Context<E> &, Symbol<E> &, u32, U32<E> *);
+
+template std::optional<ElfSym<E>>
+to_output_esym(Context<E> &, Symbol<E> &, u32, U32<E> *);
 
 } // namespace mold
