@@ -9,10 +9,10 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe1 $t/a.o -fno-PIE
-readelf -W --dyn-syms $t/exe1 | grep -Fq '__global_pointer$'
+readelf -W --dyn-syms $t/exe1 | grep -F '__global_pointer$'
 
 $CC -B. -o $t/exe2 $t/a.o -fPIE
-readelf -W --dyn-syms $t/exe2 | grep -Fq '__global_pointer$'
+readelf -W --dyn-syms $t/exe2 | grep -F '__global_pointer$'
 
 cat <<EOF | $CC -o $t/b.o -c -xc - -fPIC
 #include <stdio.h>
@@ -22,5 +22,4 @@ int hello() {
 EOF
 
 $CC -B. -o $t/c.so $t/b.o -shared
-readelf -W --dyn-syms $t/c.so > $t/log1
-! grep -Fq '__global_pointer$' $t/log1 || false
+readelf -W --dyn-syms $t/c.so | not grep -F '__global_pointer$'
