@@ -25,7 +25,7 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe1 $t/a.o -Wl,-z,lazy
-$QEMU $t/exe1 | grep -q 'Hello world'
+$QEMU $t/exe1 | grep 'Hello world'
 
 $CC -B. -o $t/exe2 $t/a.o -Wl,-z,now
-$QEMU $t/exe2 | grep -q 'Hello world'
+$QEMU $t/exe2 | grep 'Hello world'
