@@ -3,10 +3,10 @@
 
 [ $MACHINE = i686 ] && skip
 [ $MACHINE = m68k ] && skip
-[ $MACHINE = arm ] && skip
 [ $MACHINE = ppc64 ] && skip
 [ $MACHINE = ppc64le ] && skip
 [ $MACHINE = sh4 ] && skip
+[[ $MACHINE = arm* ]] && skip
 [[ $MACHINE = loongarch* ]] && skip
 
 cat <<EOF | $CC -shared -o $t/a.so -xc -
