@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2024 Intel Corporation
+    Copyright (c) 2005-2025 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -241,7 +241,12 @@ void arena::process(thread_data& tls) {
     __TBB_ASSERT(tls.my_arena == this, "my_arena is used as a hint when searching the arena to join");
 }
 
-arena::arena(threading_control* control, unsigned num_slots, unsigned num_reserved_slots, unsigned priority_level) {
+arena::arena(threading_control* control, unsigned num_slots, unsigned num_reserved_slots, unsigned priority_level
+#if __TBB_PREVIEW_PARALLEL_PHASE
+             , tbb::task_arena::leave_policy lp 
+#endif
+)
+{
     __TBB_ASSERT( !my_guard, "improperly allocated arena?" );
     __TBB_ASSERT( sizeof(my_slots[0]) % cache_line_size()==0, "arena::slot size not multiple of cache line size" );
     __TBB_ASSERT( is_aligned(this, cache_line_size()), "arena misaligned" );
@@ -276,10 +281,18 @@ arena::arena(threading_control* control, unsigned num_slots, unsigned num_reserv
     my_critical_task_stream.initialize(my_num_slots);
 #endif
     my_mandatory_requests = 0;
+
+#if __TBB_PREVIEW_PARALLEL_PHASE
+    my_thread_leave.set_initial_state(lp);
+#endif
 }
 
 arena& arena::allocate_arena(threading_control* control, unsigned num_slots, unsigned num_reserved_slots,
-                              unsigned priority_level)
+                             unsigned priority_level
+#if __TBB_PREVIEW_PARALLEL_PHASE
+                             , tbb::task_arena::leave_policy lp
+#endif
+)
 {
     __TBB_ASSERT( sizeof(base_type) + sizeof(arena_slot) == sizeof(arena), "All arena data fields must go to arena_base" );
     __TBB_ASSERT( sizeof(base_type) % cache_line_size() == 0, "arena slots area misaligned: wrong padding" );
@@ -290,7 +303,11 @@ arena& arena::allocate_arena(threading_control* control, unsigned num_slots, uns
     std::memset( storage, 0, n );
 
     return *new( storage + num_arena_slots(num_slots, num_reserved_slots) * sizeof(mail_outbox) )
-        arena(control, num_slots, num_reserved_slots, priority_level);
+        arena(control, num_slots, num_reserved_slots, priority_level
+#if __TBB_PREVIEW_PARALLEL_PHASE
+              , lp
+#endif
+        );
 }
 
 void arena::free_arena () {
@@ -443,11 +460,21 @@ void arena::enqueue_task(d1::task& t, d1::task_group_context& ctx, thread_data&
     advertise_new_work<work_enqueued>();
 }
 
-arena& arena::create(threading_control* control, unsigned num_slots, unsigned num_reserved_slots, unsigned arena_priority_level, d1::constraints constraints) {
+arena &arena::create(threading_control *control, unsigned num_slots,
+                     unsigned num_reserved_slots, unsigned arena_priority_level,
+                     d1::constraints constraints
+#if __TBB_PREVIEW_PARALLEL_PHASE
+                     , tbb::task_arena::leave_policy lp 
+#endif
+) {
     __TBB_ASSERT(num_slots > 0, NULL);
     __TBB_ASSERT(num_reserved_slots <= num_slots, NULL);
     // Add public market reference for an external thread/task_arena (that adds an internal reference in exchange).
-    arena& a = arena::allocate_arena(control, num_slots, num_reserved_slots, arena_priority_level);
+    arena& a = arena::allocate_arena(control, num_slots, num_reserved_slots, arena_priority_level
+#if __TBB_PREVIEW_PARALLEL_PHASE
+                                     , lp
+#endif
+    );
     a.my_tc_client = control->create_client(a);
     // We should not publish arena until all fields are initialized
     control->publish_client(a.my_tc_client, constraints);
@@ -500,6 +527,8 @@ struct task_arena_impl {
     static int max_concurrency(const d1::task_arena_base*);
     static void enqueue(d1::task&, d1::task_group_context*, d1::task_arena_base*);
     static d1::slot_id execution_slot(const d1::task_arena_base&);
+    static void enter_parallel_phase(d1::task_arena_base*, std::uintptr_t);
+    static void exit_parallel_phase(d1::task_arena_base*, std::uintptr_t);
 };
 
 void __TBB_EXPORTED_FUNC initialize(d1::task_arena_base& ta) {
@@ -534,6 +563,14 @@ d1::slot_id __TBB_EXPORTED_FUNC execution_slot(const d1::task_arena_base& arena)
     return task_arena_impl::execution_slot(arena);
 }
 
+void __TBB_EXPORTED_FUNC enter_parallel_phase(d1::task_arena_base* ta, std::uintptr_t flags) {
+    task_arena_impl::enter_parallel_phase(ta, flags);
+}
+
+void __TBB_EXPORTED_FUNC exit_parallel_phase(d1::task_arena_base* ta, std::uintptr_t flags) {
+    task_arena_impl::exit_parallel_phase(ta, flags);
+}
+
 void task_arena_impl::initialize(d1::task_arena_base& ta) {
     // Enforce global market initialization to properly initialize soft limit
     (void)governor::get_thread_data();
@@ -568,7 +605,12 @@ void task_arena_impl::initialize(d1::task_arena_base& ta) {
     __TBB_ASSERT(ta.my_arena.load(std::memory_order_relaxed) == nullptr, "Arena already initialized");
     unsigned priority_level = arena_priority_level(ta.my_priority);
     threading_control* thr_control = threading_control::register_public_reference();
-    arena& a = arena::create(thr_control, unsigned(ta.my_max_concurrency), ta.my_num_reserved_slots, priority_level, arena_constraints);
+    arena& a = arena::create(thr_control, unsigned(ta.my_max_concurrency), ta.my_num_reserved_slots,
+                             priority_level, arena_constraints
+#if __TBB_PREVIEW_PARALLEL_PHASE
+                             , ta.get_leave_policy()
+#endif
+    );
 
     ta.my_arena.store(&a, std::memory_order_release);
 #if __TBB_CPUBIND_PRESENT
@@ -875,6 +917,21 @@ int task_arena_impl::max_concurrency(const d1::task_arena_base *ta) {
     return int(governor::default_num_threads());
 }
 
+#if __TBB_PREVIEW_PARALLEL_PHASE
+void task_arena_impl::enter_parallel_phase(d1::task_arena_base* ta, std::uintptr_t /*reserved*/) {
+    arena* a = ta ? ta->my_arena.load(std::memory_order_relaxed) : governor::get_thread_data()->my_arena;
+    __TBB_ASSERT(a, nullptr);
+    a->my_thread_leave.register_parallel_phase();
+    a->advertise_new_work<arena::work_enqueued>();
+}
+
+void task_arena_impl::exit_parallel_phase(d1::task_arena_base* ta, std::uintptr_t flags) {
+    arena* a = ta ? ta->my_arena.load(std::memory_order_relaxed) : governor::get_thread_data()->my_arena;
+    __TBB_ASSERT(a, nullptr);
+    a->my_thread_leave.unregister_parallel_phase(/*with_fast_leave=*/static_cast<bool>(flags));
+}
+#endif
+
 void isolate_within_arena(d1::delegate_base& d, std::intptr_t isolation) {
     // TODO: Decide what to do if the scheduler is not initialized. Is there a use case for it?
     thread_data* tls = governor::get_thread_data();
