@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2021 Intel Corporation
+    Copyright (c) 2005-2024 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -28,6 +28,7 @@
 #include <cstdio>
 #include <atomic>
 
+#include "test_buffering_try_put_and_wait.h"
 
 //! \file test_sequencer_node.cpp
 //! \brief Test for [flow_graph.sequencer_node] specification
@@ -437,6 +438,169 @@ void test_deduction_guides() {
 }
 #endif
 
+#if __TBB_PREVIEW_FLOW_GRAPH_TRY_PUT_AND_WAIT
+void test_seq_node_try_put_and_wait() {
+    using namespace test_try_put_and_wait;
+
+    std::vector<int> start_work_items;
+    std::vector<int> new_work_items;
+    int wait_message = 10;
+
+    for (int i = 0; i < wait_message; ++i) {
+        start_work_items.emplace_back(i);
+        new_work_items.emplace_back(i + 1 + wait_message);
+    }
+
+    auto simple_sequencer = [](int item) { return item; };
+
+    // Test push
+    // test_buffer_push tests the graph
+    // buffer1 -> function -> buffer2 -> writer
+    //     function is a queueing serial function_node that submits new_work_items once wait_message arrives
+    //     writer is an unlimited function_node that writes an item into the processed_items vector
+    // Test steps
+    //     1. push start_work_items into the buffer1
+    //     2. buffer1.try_put_and_wait(wait_message);
+    //     3. g.wait_for_all()
+    // test_buffer_push returns the index from which the items processed during wait_for_all() starts
+    {
+        std::vector<int> processed_items;
+
+        std::size_t after_start = test_buffer_push<tbb::flow::sequencer_node<int>>(start_work_items, wait_message,
+                                                                                   new_work_items, processed_items,
+                                                                                   simple_sequencer);
+
+        // Expected effect:
+        // During buffer1.try_put_and_wait()
+        //     1. start_work_items would be pushed to buffer1
+        //     2. wait_message would be pushed to buffer1
+        //     3. forward_task on buffer1 would transfer all of the items to the function_node in sequencer order (FIFO)
+        //     4. the first item would occupy concurrency of function, other items would be pushed to the queue
+        //     5. function would process start_work_items and push them to the buffer2
+        //     6. wait_message would be processed last and add new_work_items to buffer1
+        //     7. forward_task on buffer2 would transfer start_work_items in sequencer (FIFO) order and the wait_message to the writer
+        //     8.  try_put_and_wait exits since wait_message is completed
+        // During g.wait_for_all()
+        //     10. forward_task for new_work_items in buffer1 would be spawned and put items in function in FIFO order
+        //     11. function_node would process and push forward items from the queue in FIFO order
+        // Expected items processing - { start_work_items FIFO, wait_message, new_work_items FIFO }
+
+        std::size_t check_index = 0;
+        CHECK_MESSAGE(after_start == start_work_items.size() + 1,
+                      "try_put_and_wait should process start_work_items and the wait_message");
+        for (auto item : start_work_items) {
+            CHECK_MESSAGE(processed_items[check_index++] == item,
+                          "try_put_and_wait should process start_work_items FIFO");
+        }
+
+        CHECK_MESSAGE(processed_items[check_index++] == wait_message,
+                      "try_put_and_wait should process wait_message after start_work_items");
+
+        for (auto item : new_work_items) {
+            CHECK_MESSAGE(processed_items[check_index++] == item,
+                          "wait_for_all should process new_work_items FIFO");
+        }
+        CHECK(check_index == processed_items.size());
+    } // Test push
+
+    // Test pull
+    // test_buffer_pull tests the graph
+    // buffer -> function
+    //     function is a rejecting serial function_node that submits new_work_items once wait_message arrives
+    //     and writes the processed item into the processed_items
+    // Test steps
+    //     1. push the occupier message to the function
+    //     2. push start_work_items into the buffer
+    //     3. buffer.try_put_and_wait(wait_message)
+    //     4. g.wait_for_all()
+    // test_buffer_pull returns the index from which the items processed during wait_for_all() starts
+
+    {
+        std::vector<int> processed_items;
+        int occupier = 42;
+
+        std::size_t after_start = test_buffer_pull<tbb::flow::sequencer_node<int>>(start_work_items, wait_message, occupier,
+                                                                                   new_work_items, processed_items,
+                                                                                   simple_sequencer);
+
+        // Expected effect
+        // 0. task for occupier processing would be spawned by the function
+        // During buffer.try_put_and_wait()
+        //     1. start_work_items would be pushed to the buffer
+        //     2. wait_message would be pushed to the buffer
+        //     3. forward_task would try to push items to the function, but would fail
+        //        and set the edge to the pull state
+        //     4. occupier would be processed
+        //     5. items would be taken from the buffer by function in FIFO order
+        //     6. wait_message would be taken last and push new_work_items to the buffer
+        // During wait_for_all()
+        //     7. new_work_items would be taken from the buffer in FIFO order
+        // Expected items processing { occupier, start_work_items FIFO, wait_message, new_work_items FIFO }
+
+        std::size_t check_index = 0;
+
+        CHECK_MESSAGE(after_start == start_work_items.size() + 2,
+                      "start_work_items, occupier and wait_message should be processed by try_put_and_wait");
+        CHECK_MESSAGE(processed_items[check_index++] == occupier, "Unexpected items processing by try_put_and_wait");
+        for (auto item : start_work_items) {
+            CHECK_MESSAGE(processed_items[check_index++] == item,
+                          "try_put_and_wait should process start_work_items FIFO");
+        }
+        CHECK_MESSAGE(processed_items[check_index++] == wait_message, "Unexpected items processing by try_put_and_wait");
+
+        for (auto item : new_work_items) {
+            CHECK_MESSAGE(processed_items[check_index++] == item,
+                          "try_put_and_wait should process new_work_items FIFO");
+        }
+        CHECK(check_index == processed_items.size());
+    }
+
+    // Test reserve
+    {
+        int thresholds[] = { 1, 2 };
+
+        for (int threshold : thresholds) {
+            std::vector<int> processed_items;
+
+            // test_buffer_reserve tests the following graph
+            // buffer -> limiter -> function
+            //  function is a rejecting serial function_node that puts an item to the decrementer port
+            //  of the limiter inside of the body
+
+            std::size_t after_start = test_buffer_reserve<tbb::flow::sequencer_node<int>>(threshold,
+                start_work_items, wait_message, new_work_items, processed_items, simple_sequencer);
+
+            // Expected effect:
+            // 1. start_work_items would be pushed to the buffer
+            // 2. wait_message_would be pushed to the buffer
+            // 3. forward task of the buffer would push the first message to the limiter node.
+            //    Since the limiter threshold is not reached, it would be directly passed to the function
+            // 4. function would spawn the task for the first message processing
+            // 5. the first would be processed
+            // 6. decrementer.try_put() would be called and the limiter node would
+            //    process all of the items from the buffer using the try_reserve/try_consume/try_release semantics
+            // 7. When the wait_message would be taken from the buffer, the try_put_and_wait would exit
+
+            std::size_t check_index = 0;
+
+            CHECK_MESSAGE(after_start == start_work_items.size() + 1,
+                      "start_work_items, occupier and wait_message should be processed by try_put_and_wait");
+            for (auto item : start_work_items) {
+                CHECK_MESSAGE(processed_items[check_index++] == item,
+                            "try_put_and_wait should process start_work_items FIFO");
+            }
+            CHECK_MESSAGE(processed_items[check_index++] == wait_message, "Unexpected items processing by try_put_and_wait");
+
+            for (auto item : new_work_items) {
+                CHECK_MESSAGE(processed_items[check_index++] == item,
+                            "try_put_and_wait should process new_work_items FIFO");
+            }
+            CHECK(check_index == processed_items.size());
+        }
+    }
+}
+#endif // __TBB_PREVIEW_FLOW_GRAPH_TRY_PUT_AND_WAIT
+
 //! Test sequencer with various request orders and parallelism levels
 //! \brief \ref requirement \ref error_guessing
 TEST_CASE("Serial and parallel test"){
@@ -501,3 +665,10 @@ TEST_CASE("constraints for sequencer_node sequencer") {
     static_assert(!can_call_sequencer_node_ctor<type, WrongReturnOperatorRoundBrackets<type>>);
 }
 #endif // __TBB_CPP20_CONCEPTS_PRESENT
+
+#if __TBB_PREVIEW_FLOW_GRAPH_TRY_PUT_AND_WAIT
+//! \brief \ref error_guessing
+TEST_CASE("test sequencer_node try_put_and_wait") {
+    test_seq_node_try_put_and_wait();
+}
+#endif
